@@ -281,8 +281,6 @@ struct pull_target {
 	/*@null@*/struct pull_source *sources;
 	/*@dependent@*/struct target *target;
 	/*@null@*/struct upgradelist *upgradelist;
-	/* Ignore delete marks (as some lists were missing) */
-	bool ignoredelete;
 };
 
 static void pull_freetargets(struct pull_target *targets) {
@@ -380,7 +378,6 @@ static retvalue generatepulltarget(struct pull_distribution *pd, struct target *
 	pt->target = target;
 	pt->next = pd->targets;
 	pt->upgradelist = NULL;
-	pt->ignoredelete = false;
 	pt->sources = NULL;
 	s = &pt->sources;
 	pd->targets = pt;
@@ -708,7 +705,6 @@ static inline retvalue pull_searchformissing(/*@null@*/FILE *out,struct database
 			RET_UPDATE(result,r);
 			if( RET_WAS_ERROR(r) )
 				return result;
-			p->ignoredelete = false;
 			continue;
 		}
 
@@ -750,6 +746,23 @@ static retvalue pull_search(/*@null@*/FILE *out,struct database *database,struct
 	return result;
 }
 
+static bool pull_isbigdelete(struct pull_distribution *d) {
+	struct pull_target *u, *v;
+
+	for( u = d->targets ; u != NULL ; u=u->next ) {
+		if( upgradelist_isbigdelete(u->upgradelist) ) {
+			d->distribution->omitted = true;
+			for( v = d->targets ; v != NULL ; v = v->next ) {
+				upgradelist_free(v->upgradelist);
+				v->upgradelist = NULL;
+			}
+			return true;
+		}
+	}
+	return false;
+}
+
+
 static void pull_from_callback(void *privdata, const char **rule_p, const char **from_p) {
 	struct pull_source *source = privdata;
 
@@ -767,7 +780,7 @@ static retvalue pull_install(struct database *database, struct pull_distribution
 	result = RET_NOTHING;
 	for( u=distribution->targets ; u != NULL ; u=u->next ) {
 		r = upgradelist_install(u->upgradelist, d->logger,
-				database, u->ignoredelete,
+				database, false,
 				pull_from_callback);
 		RET_UPDATE(d->status, r);
 		RET_UPDATE(result,r);
@@ -926,6 +939,14 @@ retvalue pull_update(struct database *database, struct pull_distribution *distri
 		printf("Installing (and possibly deleting) packages...\n");
 
 	for( d=distributions ; d != NULL ; d=d->next) {
+		if( global.onlysmalldeletes ) {
+			if( pull_isbigdelete(d) ) {
+				fprintf(stderr,
+"Not processing '%s' because of --onlysmalldeletes\n",
+						d->distribution->codename);
+				continue;
+			}
+		}
 		r = pull_install(database, d);
 		RET_UPDATE(result,r);
 		if( RET_WAS_ERROR(r) )
