@@ -8,7 +8,7 @@ struct uploaders;
 
 enum upload_condition_type { uc_REJECTED = 0, uc_ALWAYS,
 /*	uc_COMPONENT, */ uc_ARCHITECTURES,
-	uc_SOURCENAME, uc_SECTIONS, uc_BINARIES };
+	uc_SOURCENAME, uc_SECTIONS, uc_BINARIES, uc_BYHAND };
 #define uc_ACCEPTED uc_ALWAYS
 
 retvalue uploaders_get(/*@out@*/struct uploaders **list, const char *filename);
